% Julio Usaola García
% Universidad Carlos III de Madrid.
% precios zonales
% las variables son las potencias generadas y demandadas y las de la
% interconexión

clear variables
close all
clc
format compact

% variables
% pg1, pg2, pg3, pd1, pd2, pd3, p12, p13, p23
% número de variables: 3 potencias generadas, 3 potencias demandadas y 
% 3 potencias por las interconexiones
nvar=9; 

pmax=10000;  % máxima potencia de la intcnx

% inicialización de matrices
H=zeros(nvar);
f=zeros(nvar,1);

% matriz de coeficientes cuadráticos (diagonal)
H(1,1)=1.83e-3; % pg1
H(2,2)=1.25e-3; % pg2
H(3,3)=2.25e-3; % pg3
H(4,4)=11e-3; % pd1
H(5,5)=10e-3; % pd2
H(6,6)=9e-3; % pd3

% vector de coeficientes lineales (solo para la función de demanda)
f(4)=-385;  % pd1
f(5)=-450;  % pd2
f(6)=-225;  % pd3

% matriz de restricciones de igualdad
Aeq=[
    .45375 -.18125 0 -.45375  .18125  0   -1    0    0
    .54500  .18200 0 -.54500 -.18200  0    0   -1    0
    .45500  .81750 0 -.45500 -.81750  0    0    0   -1
       -1     -1  -1     1       1    1    0    0    0
       ];
%      pg1    pg2 pg3 | pd1    pd2  pd3 | p12 p13  p23

Beq=zeros(size(Aeq,1),1);  % término independiente Beq --> nulo

% límites superiores e inferiores de las variables
lb=zeros(nvar,1);
lb(7)=-pmax;
lb(8)=-pmax;
lb(9)=-pmax;
ub=Inf(nvar,1);
ub(7)=pmax;
ub(8)=pmax;
ub(9)=pmax;

[x,fval,exitflag,output,lambda]= quadprog(H,f,[],[],Aeq,Beq,lb,ub);

% x(1)...x(3) son las potencias generadas en cada zona
% x(4)...x(6) son las potencias demandadas en cada zona
% x(7)...x(9) son las potencias circulantes por las interconexiones

% cálculo de precios
lam1g=H(1,1)*x(1);   % precio en zona 1 a partir de la funci�n de oferta de la zona
lam1d=-f(4)-H(4,4)*x(4); % precio en zona 2 a partir de la funci�n de demanda
% para las restantes zonas:
lam2g=H(2,2)*x(2);
lam2d=-f(5)-H(5,5)*x(5);
lam3g=H(3,3)*x(3);
lam3d=-f(6)-H(6,6)*x(6);

% presentación de resultados
% NOTA: 'num2str' convierte una variable numérica en una variable de 
% caracteres y facilita la presentación de resultados

potgen=['Pg1= ' num2str(x(1)) '      Pg2= ' num2str(x(2)) '     Pg3= ' num2str(x(3))];
potdem=['Pd1= ' num2str(x(4)) '      Pd2= ' num2str(x(5)) '     Pd3= ' num2str(x(6))];
pram=x(7:9);
potcnx=['P12= ' num2str(pram(1)) '      P13= ' num2str(pram(2)) '     P23= ' num2str(pram(3))];




precio=['pi1g= ' num2str(lam1g) '   pi1d= ' num2str(lam1d)...
    '  pi2g= ' num2str(lam2g) '   pi2d= ' num2str(lam2d)...
    '  pi3g= ' num2str(lam3g) '   pi3d= ' num2str(lam3d)];


disp(potgen)
disp(potdem)
disp(potcnx)
disp(precio)

